import time
from layout import *
from log import *

def date(t):
    y, l, d, h, m, s, wd, yd, dst = time.localtime(t)
    return y, l, d

def weekday(t):
    y, l, d, h, m, s, wd, yd, dst = time.localtime(t)
    return wd

def year(y):
    return int(time.mktime((y, 1, 1, 0, 0, 0, 0, 0, 0)))

def monthday(y, l):
    t = int(time.mktime((y, l, 1, 0, 0, 0, 0, 0, 0)))
    y, l, d, h, m, s, wd, yd, dst = time.localtime(t)
    return yd - 1

class LogScanner:
    def __init__(self, log):
        self.log = log
        self.i = 0

    def get(self, min_time, max_time):
        if max_time <= self.log[0][0] or min_time >= self.log[-1][0]:
            return []
        i = self.i
        while 1:
            start, action = self.log[i]
            if i + 1 >= len(self.log):
                end, action = max_time, ''
            else:
                end, next = self.log[i + 1]
            if end <= min_time:
                i += 1
                continue
            if start > min_time:
                i -= 100
                continue
            break
        entries = [(max(start, min_time), action, min(end, max_time))]
        while end < max_time:
            i += 1
            start, action = self.log[i]
            if i + 1 >= len(self.log):
                end, action = max_time, ''
            else:
                end, next = self.log[i + 1]
            entries.append((start, action, min(end, max_time)))
        self.i = i
        return entries
            
def get_colour(action):
    if 'sleep' in action or 'nap' in action:
        return Color(0.6, 0.6, 0.6)
    elif ('thesis' in action or 'dissertation' in action or 'talk' in action or
          'paper' in action or 'writing' in action or 'editing' in action or
          'ttb' in action or 'work' in action or 'meet' in action or
          'meeting' in action or 'admin' in action):
        return Color(0.25, 0.5, 1)
    elif 'dev' in action or 'hack' in action:
        return Color(1, 0, 0.25)
    elif 'kng' in action:
        return Color(0.25, 0.75, 0.25)
    elif ('fun' in action or 'party' in action or 'play' in action or
          'sex' in action or 'cuddle' in action or 'chat' in action or
          'hang' in action or 'watch' in action):
        return Color(1, 0.95, 0.4)
    elif ('web' in action or 'dailykos' in action or 'lj' in action or
          'crooksandliars' in action or 'slashdot' in action or
          'news' in action or 'blogs' in action or 'blog' in action or
          'e-mail' in action):
        return Color(0.8, 0.25, 1)

def get_thesis(action):
    if 'sleep' in action or 'nap' in action:
        return Color(0.6, 0.6, 0.6)
    elif 'thesis' in action or 'dissertation' in action:
        return Color(0.25, 0.5, 1)
    elif 'ttb' in action:
        return Color(0.8, 0.5, 0.25)
    elif 'thesis' in action or 'dissertation' in action:
        if 'talk' in action:
            return Color(0.75, 0.25, 0.9)
        return Color(1, 0.65, 0.25)

def line_cell(t, index):
    return index, 0

def week_cell(t, index):
    i = index + weekday(t - index*24*3600)
    return int(i/7), i%7

log = Log('/home/ping/.timetrack', 10000)
scanner = LogScanner(log)

def chart((vx, vy), (ox, oy), (min, max),
          get_cell, get_colour=get_colour, (gx, gy)=(0, 0), step=24*3600):
    box = Box()
    index = 0
    for t in range(min, max, step):
        hi, lo = get_cell(t, index)
        x = vx*lo + ox*hi
        y = vy*lo + oy*hi
        for start, action, end in scanner.get(t, t + step):
            fill = get_colour(action.split())
            if fill:
                w = ox - gx + vx*(end - start)/float(step)
                h = oy - gy + vy*(end - start)/float(step)
                rx = x + vx*(start - t)/float(step)
                ry = y + vy*(start - t)/float(step)
                box.add(Rect(w, h, weight=0, fill=fill), (rx, ry), 'nw')
        index += 1
    return box

monthabbrs = 'x Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec'.split()

def plot(y, month_start, month_end, width, padding=0, show_action=lambda x: 1):
    g = Box(padding=10)
    s = Color(0.7, 0.7, 0.85)
    t = Color(0.4, 0.4, 0.7)
    start = year(y) + monthday(y, month_start)*24*3600
    if month_end == 12:
        end = year(y + 1)
    else:
        end = year(y) + monthday(y, month_end + 1)*24*3600
    cw = (end - start)/(24*3600)*width
    ch = 192
    Label = curry(Text, font=Font('LucidaGrande.ttf'), size=9, fill=t)
    c = chart((0, ch), (width, 0), (start, end), line_cell,
              lambda action: get_colour(filter(show_action, action)),
              (padding, 0))
    for l in range(month_start, month_end + 1):
        x = (monthday(y, l) - monthday(y, month_start))*width
        g.add(Line(0, ch + 12, stroke=s), (x + 0.5, -12))
        g.add(Label(monthabbrs[l]), (x + 2, -11), 'nw')
    g.add(Line(0, ch + 12, stroke=s), (cw + 0.5, -12))
    x = 0
    for d in range(start, end + 1, 24*3600):
        if weekday(d) == 6 and x not in [0, cw]:
            g.add(Line(0, ch + 4, stroke=s, weight=0.5), (x + 0.5, -4))
        x += width
    for h in range(0, 25, 6):
        y = h*8
        wt = 1.0 - ((h/6)%2)*0.5
        g.add(Line(cw + 24, 0, stroke=s, weight=wt), (-24, y + 0.5))
        g.add(Label(str(h)), (-23, y + 2), 'nw')
    g.add(c)
    return g

def thesis_only(x):
    return x in ['nap', 'sleep', 'thesis', 'dissertation', 'talk']

def sleep_only(x):
    return x in ['nap', 'sleep']

def write_plots():
    plot(2006, 1, 12, 2, 0).write('2006.png')
    plot(2006, 1, 12, 2, 0, sleep_only).write('2006.sleep.png')
    plot(2007, 1, 12, 2, 0).write('2007.png')
    plot(2007, 1, 12, 2, 0, sleep_only).write('2007.sleep.png')
    plot(2007, 1, 6, 5, 1).write('2007.01-06.png')
    plot(2007, 1, 6, 5, 1, sleep_only).write('2007.01-06.sleep.png')
    plot(2007, 7, 12, 5, 1).write('2007.07-12.png')
    plot(2007, 7, 12, 5, 1, sleep_only).write('2007.07-12.sleep.png')
    plot(2007, 7, 12, 5, 1, thesis_only).write('2007.07-12.thesis.png')
    plot(2007, 8, 12, 5, 1, thesis_only).write('2007.08-12.thesis.png')
    plot(2008, 1, 6, 5, 1).write('2008.01-06.png')
    plot(2008, 1, 6, 5, 1, sleep_only).write('2008.01-06.sleep.png')

def write(self, filename, *args, **kw):
    Block.write(self, filename, *args, **kw)
    print filename
Box.write = write

plot(2008, 1, 3, 5, 1).write('2008.01-03.png')
plot(2008, 1, 3, 5, 1, sleep_only).write('2008.01-03.sleep.png')
